#!/usr/bin/ruby
# -*- coding: utf-8 -*-

# WordPressImageUploader
# Ver.0.1
# 2014/04/02 moyashi http://hitoriblog.com/

# ↓こちらのスクリプトを改造しました
# http://www.goodpic.com/mt/archives2/2008/03/xmlrpc_apiruby.html
# 参考1: https://github.com/kmuto/post-xmlrpc/blob/master/post-xmlrpc.rb
# 参考2: http://uisteven.blog.fc2.com/blog-entry-44.html

require "xmlrpc/client"
require 'yaml'
require 'kconv'
require 'shellwords'

# ---------------------------------------------------------------
# Picture
# ---------------------------------------------------------------

class Picture
  def initialize(path)
    @path = path
    @width = property("pixelWidth").to_i
    @height = property("pixelHeight").to_i
  end

  def property(prop)
    res = `sips --getProperty #{prop} #{@path.shellescape}`.toutf8
    match = res.match(/#{prop}: ([0-9]+)/)
    if match.length > 1
      return match[1]
    else
      return nil 
    end
  end

  def width()
    return @width
  end
  
  def height()
    return @height
  end

  def portrait?()
    if @width > @height
      return false
    else
      return true
    end
  end
end

# ---------------------------------------------------------------
# WordPressImageUploader
# ---------------------------------------------------------------

class WordPressImageUploader

  def initialize()
    begin
      config_file = bundle_name + ".yaml"
      @config = YAML.load_file(File.join(install_dir(), config_file))
    rescue XMLRPC::FaultException => e
      puts "fault #{e.faultCode}: #{e.faultString}"
    end
    end_point = @config['end_point']
    @username = @config['account']
    @password = @config['password']
    @blog_id = @config['blog_id']
    @upload_dir = @config['upload_dir']
    @temp_dir = @config['temp_dir']
    @long_side = @config['long_side']
    @format = @config['format']
    @convert_path = @config['convert_path']
    @convert_options = @config['convert_options']

    client = XMLRPC::Client.new2( end_point )

    # Ruby 2.0 workaround
    client.http_header_extra = {'accept-encoding' => 'identity'}

    @meta_weblog = client.proxy("metaWeblog")
  end

  def bundle_name()
    return File.basename(File.expand_path(File.join(File.dirname(__FILE__), "../../../")), ".*")
  end

  def install_dir()
    return File.expand_path(File.join(File.dirname(__FILE__), "../../../../"))
  end

  def install_dir2()
    return File.expand_path(File.dirname(__FILE__))
  end

  def format()
    return @format
  end

  def convert_path()
    return @convert_path
  end

  def convert_options()
    if @convert_options.nil?
      return ""
    else
      return @convert_options
    end
  end

  def temp_dir()
    return @temp_dir
  end

  def long_side()
    return @long_side
  end

  def file_type(file_path)
    file_types = {
      "JPG" => "image/jpeg",
      "jpg" => "image/jpeg",
      "JPEG" => "image/jpeg",
      "jpeg" => "image/jpeg",
      "png" => "image/png",
      "PNG" => "image/png",
      "GIF" => "image/gif",
      "gif" => "image/gif"
    }
    ext = File.extname(file_path).gsub(".", "")
    if (ext.empty?)
      return "application/octet-stream"
    else
      type = file_types[ext]
      if type.empty?
        return "application/octet-stream"
      else
        return type
      end
    end
  end

  # XMLRPCでファイルをアップロード
  def upload(file_path)
    if File.file?(file_path) then
      file_name = File.basename(file_path)
      content = File.open(file_path,"rb").readlines.join('')
      base64 = XMLRPC::Base64.new(content)
      file_hash = {:name => file_name, :bits => base64, :type => self.file_type(file_name)}
      begin
        result = @meta_weblog.newMediaObject @blog_id, @username, @password, file_hash
        return result
      rescue XMLRPC::FaultException => e
        puts "fault #{e.faultCode}: #{e.faultString}"
      end   
    
    else
      puts "#{file_path} is not a file"
    end
  
  end

end

# ---------------------------------------------------------------
# Util
# ---------------------------------------------------------------

module Util
  def self.pbcopy(str)
    IO.popen("pbcopy", "r+") do |io|
      io.puts str.tosjis
      io.close_write
      io.gets
    end
  end
end

# ---------------------------------------------------------------
# Main
# ---------------------------------------------------------------

uploader = WordPressImageUploader.new()

imgs = []

ARGV.each do |file_path|
  pic = Picture.new(file_path)
  temp_dir = File.expand_path(uploader.temp_dir)
  if !FileTest.exists?(temp_dir) && !FileTest.directory?(temp_dir)
    Dir.mkdir(temp_dir)
  end

  file_name = File.basename(file_path)
  temp_file = File.join(temp_dir, file_name)
  temp_pic = nil

  if pic.portrait?
    if pic.height.to_i > uploader.long_side.to_i
      if (uploader.convert_path.nil?)
        system("sips -s formatOptions 80 --resampleHeight #{uploader.long_side} #{file_path.shellescape} --out #{temp_file.shellescape} >/dev/null 2>&1")
      else
        system("#{uploader.convert_path} -geometry #{uploader.long_side}x#{uploader.long_side} #{uploader.convert_options} #{file_path.shellescape} #{temp_file.shellescape} >/dev/null 2>&1")
      end
      result = uploader.upload(temp_file)
      temp_pic = Picture.new(temp_file)
    else
      result = uploader.upload(file_path)
    end
  else
    if pic.width.to_i > uploader.long_side.to_i
      if (uploader.convert_path.nil?)
        system("sips -s formatOptions 80 --resampleWidth #{uploader.long_side} #{file_path.shellescape} --out #{temp_file.shellescape} >dev/null >/dev/null 2>&1")
      else
        system("#{uploader.convert_path} -geometry #{uploader.long_side}x#{uploader.long_side} #{uploader.convert_options} #{file_path.shellescape} #{temp_file.shellescape} >/dev/null 2>&1")
      end
      result = uploader.upload(temp_file)
      temp_pic = Picture.new(temp_file)
    else
      result = uploader.upload(file_path)
    end
  end

  url = result['url']
  filename = File.basename(result['url'])
  basename = File.basename(result['url'], ".*")
 
 if (temp_pic)
    width = temp_pic.width
    height = temp_pic.height
  else
    width = pic.width
    height = pic.height
  end
  imgs.push(eval('%Q('+uploader.format+')'))
end

if imgs.length > 0
  Util.pbcopy(imgs.join("\n"))
end
